"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.packageSync = void 0;
const fs_1 = require("fs");
const packageSync = () => {
    const pack = (0, fs_1.readFileSync)("package.json", "utf-8");
    const packJson = JSON.parse(pack);
    const packSvelte = (0, fs_1.readFileSync)("package.svelte.json", "utf-8");
    const svelte = JSON.parse(packSvelte);
    const packVue = (0, fs_1.readFileSync)("package.vue.json", "utf-8");
    const vue = JSON.parse(packVue);
    const packReact = (0, fs_1.readFileSync)("package.react.json", "utf-8");
    const react = JSON.parse(packReact);
    const svelteWarnings = [];
    const vueWarnings = [];
    const reactWarnings = [];
    Object.keys(packJson.dependencies).map((dep) => {
        const version = packJson.dependencies[dep];
        const svelteVersion = svelte.dependencies[dep];
        const vueVersion = vue.dependencies[dep];
        const reactVersion = react.dependencies[dep];
        if (svelteVersion && svelteVersion !== version) {
            svelteWarnings.push(`svelte: ${dep} ${svelteVersion} -> ${version}`);
        }
        if (vueVersion && vueVersion !== version) {
            vueWarnings.push(`vue: ${dep} ${vueVersion} -> ${version}`);
        }
        if (reactVersion && reactVersion !== version) {
            reactWarnings.push(`react: ${dep} ${reactVersion} -> ${version}`);
        }
    });
    Object.keys(packJson.devDependencies).map((dep) => {
        const version = packJson.devDependencies[dep];
        const svelteVersion = svelte.devDependencies[dep];
        const vueVersion = vue.devDependencies[dep];
        const reactVersion = react.devDependencies[dep];
        if (svelteVersion && svelteVersion !== version) {
            svelteWarnings.push(`svelte (devDependencies): ${dep} ${svelteVersion} -> ${version}`);
        }
        if (vueVersion && vueVersion !== version) {
            vueWarnings.push(`vue (devDependencies): ${dep} ${vueVersion} -> ${version}`);
        }
        if (reactVersion && reactVersion !== version) {
            reactWarnings.push(`react (devDependencies): ${dep} ${reactVersion} -> ${version}`);
        }
    });
    console.log("Svelte Warnings", svelteWarnings);
    console.log("Vue Warnings", vueWarnings);
    console.log("React Warnings", reactWarnings);
};
exports.packageSync = packageSync;
